# -*- coding: utf-8 -*-
"""Functions from market data"""

__author__ = "Miguel Martin"
__version__ = "1"


class TestSummary:
    def __init__(self, system_desc, trades_desc):
        self.system_stats = system_desc
        self.trades_stats = trades_desc
